#include <vector>
#include <iostream>
#include <algorithm>
#include <cmath>
#include <array>

using namespace std;
typedef long double ld;

#ifdef DEBUG 
#define D_LOG(x) x
#else 
#define D_LOG(x)
#endif

#define DELTA 1e-9L

// Note. I don't have a TCR of my own, so I've borrowed some geometric functions from Timon.

struct vec_xy {
	ld x,y;
	vec_xy(ld new_x, ld new_y) : x(new_x), y(new_y) {}
	vec_xy() : x(0), y(0) {}

	vec_xy operator-(const vec_xy &xy) const { return vec_xy{x-xy.x, y-xy.y}; }
	vec_xy operator+(const vec_xy &xy) const { return vec_xy{x+xy.x, y+xy.y}; }
	ld operator*(const vec_xy &p) const { return x*p.x + y*p.y; }
	vec_xy operator*(ld c) const { return vec_xy{x*c, y*c}; }
	vec_xy operator/(ld c) const { return vec_xy{x/c, y/c}; }
	ld lsq() { return x*x + y*y; }
};

struct circle {
	vec_xy c;
	ld r;

	circle(ld centre_x, ld centre_y, ld radius) : c(vec_xy(centre_x, centre_y)), r(radius){
	}
};


bool equal_triple(array<ld, 3> a, array<ld, 3> b) {
	return (a[0] == b[0]) && (a[1] == b[1]) && (a[2] == b[2]);
}

bool compare_triple(const array<ld, 3> a, const array<ld, 3> b) {
	for(int i =0; i < 3; ++i) {
		if (a[i] != b[i]) {
			return a[i] < b[i];
		}
	}
	return false;
}

ld pow2(ld x) { return x*x; }

// geometry

// distance between two points.
ld distance(vec_xy a, vec_xy b) {
	return sqrt(pow2(a.x - b.x) + pow2(a.y - b.y));
}

// given two circles, if (they are equal or) the first includes the second, result true.
bool first_includes(circle a, circle b) {
	return (distance(a.c, b.c) + b.r - a.r) < DELTA;
}

// given two circles, do they overlap.
bool overlap(circle a, circle b) {
	return first_includes(a, b) || first_includes(b, a) || (distance(a.c, b.c) - b.r - a.r < DELTA);
}

// Do two circles, intersect, (Timon)
vector<vec_xy> intersect(circle c1, circle c2) {
	vector<vec_xy> result = vector<vec_xy>(); 
	// Precompute some terms.
	vec_xy d = c1.c - c2.c;
	ld dsq = d.lsq();
	ld det = -(dsq - pow2(c1.r + c2.r)) * (dsq - pow2(c1.r-c2.r));
	vec_xy m = d * (pow2(c2.r) - pow2(c1.r) + dsq);

	// Single intersection point.
	if (det < DELTA) {
		result.push_back(c2.c + m / (2.0 * dsq));
	} else {
		// Two intersection points.
		ld sdet = sqrt(det);
		result.push_back(vec_xy{
			c2.c.x + (m.x - d.y * sdet) / (2.0 * dsq),
			c2.c.y + (m.y + d.x * sdet) / (2.0 * dsq)
		});
		result.push_back(vec_xy{
			c2.c.x + (m.x + d.y * sdet) / (2.0 * dsq),
			c2.c.y + (m.y - d.x * sdet) / (2.0 * dsq)
		});
	}
	return result;
}

bool contains_point(circle a, vec_xy p) {
	return (distance(p, a.c) - a.r) < DELTA;	
}

// Project a point onto a line (Timon)
vec_xy project(vec_xy l1, vec_xy l2, vec_xy p) {
	vec_xy u = l2 - l1;
	return l1 + (u * ((u * (p - l1)) / u.lsq()));
}
// helper functions.

// Generate all arrangements of the partitions, but delete duplicates.
vector<array<ld, 3>> getPartitions(
	vector<ld>::iterator s,
	vector<ld>::iterator t) {
	if (s == t) {
		return {{0,0,0}};
	} else {
		vector<array<ld, 3>> result; 
		vector<array<ld, 3>> smaller = getPartitions(s+1, t);
		for(auto it = smaller.begin(); it < smaller.end(); it++) {
			result.push_back({(*it)[0] + *s, (*it)[1], (*it)[2]});
			result.push_back({(*it)[0], (*it)[1] + *s, (*it)[2]});
			result.push_back({(*it)[0], (*it)[1], (*it)[2] + *s});
		}
		sort(result.begin(), result.end(), compare_triple);
		D_LOG(cout<<"result before resize is: "<<result.size()<<"\n";)
		auto end_unique = std::unique(result.begin(), result.end(), equal_triple);
		result.resize(std::distance(result.begin(),end_unique));
		D_LOG(cout<<"result after resize is: "<<result.size()<<"\n";)
		return result;
	}
}

bool dracula_shaded(array<ld, 3> partition, vec_xy *points) {
	D_LOG(cout<<"---- SHADED ---- partition: "<<partition[0]<<", "<<partition[1]<<", "<<partition[2]<<"\n";)
	array<circle, 3> groups = {
		circle(points[0].x, points[0].y, partition[0]),
		circle(points[1].x, points[1].y, partition[1]),
		circle(points[2].x, points[2].y, partition[2]),
	};

	vector<array<int, 3>> combinations = {
		{0, 1, 2},
		{1, 2, 0},
		{2, 0, 1}
	};

	// Of the three circles, if any is completely covered by the other, we must use the smaller of the two
	for (auto c_it = combinations.begin(); c_it < combinations.end(); c_it++) {
		if (first_includes(groups[(*c_it)[0]], groups[(*c_it)[1]]) || first_includes(groups[(*c_it)[0]], groups[(*c_it)[2]])) {
			D_LOG(cout<<"circle "<<(*c_it)[0]<<" includes one of the others\n";)
			return overlap(groups[(*c_it)[1]], groups[(*c_it)[2]]);
		}
	}	

	// Now that we have got here, we know we need to intersect all three and we know none contains one of the others.
	D_LOG(cout<<"checking all three\n";)
	if (!overlap(groups[0], groups[1]) || !overlap(groups[0], groups[2]) || !overlap(groups[1], groups[2])) {
		D_LOG(cout<<"they don't all overlap\n");	
		return false;
	}
	D_LOG(cout<<"they overlap, do they ALL overlap?\n");
	// work out where these two circles intersect.
	for (auto c_it = combinations.begin(); c_it < combinations.end(); c_it++) {
		vector<vec_xy> points = intersect(groups[(*c_it)[0]], groups[(*c_it)[1]]);

		if ( contains_point(groups[(*c_it)[2]], points[0]) ) {
			return true;
		}
		if (points.size() == 2) {
			// we have two points. As per Timon here. check either end of the linee then project.
			if ( contains_point(groups[(*c_it)[2]], points[1]) ) {
				return true;
			}

			vec_xy proj = project(points[0], points[1], groups[(*c_it)[2]].c);
			// check this point is inside all three circles.
			bool inside = true;
			inside = inside & contains_point(groups[0], proj);
			inside = inside & contains_point(groups[1], proj);
			inside = inside & contains_point(groups[2], proj);
			return inside;
		}
	}
	return false;
}

int main(){
    bool possible = false;
    // Read in the data.
	vec_xy interest[3];
	for(int i = 0; i < 3; ++i) {
		cin>>interest[i].x;
		cin>>interest[i].y;
	}
	vector<ld> diameters;
	int tree_count; cin>>tree_count;
	D_LOG(cout<<"Trees :"<<tree_count<<'\n';)
	for (int i = 0; i < tree_count; ++i) {
		ld input; cin>>input; 
		diameters.push_back(input * 2);
	}

	// sort the trees by size.
	sort(diameters.begin(), diameters.end());
	ld largest_radius = diameters.back() / 2;
	// move it off the list.
	diameters.pop_back();
	D_LOG(cout<<"largest tree radii: "<<largest_radius<<"\n";)

	// if we have one trees, the points have to all be within it.
	if (tree_count == 1) {
		possible = dracula_shaded({largest_radius, largest_radius, largest_radius}, interest);
	} else {
		auto bob = getPartitions(diameters.begin(), diameters.end());
	    for(auto it=bob.begin() ; it < bob.end(); it++) {
	    	possible = dracula_shaded(
	    		{
	    			(*it)[0] + largest_radius,
	    			(*it)[1] + largest_radius,
	    			(*it)[2] + largest_radius
	    		}, interest);
	    	if (possible) {
	    		break;
	    	}
	    }	    	
	}
	cout<<(possible ? "" : "im")<<"possible\n";
}