#include <bits/stdc++.h>
using namespace std;

// O*(3^n) enumeration.
// I intentionally used a somewhat different intersection check than the model
// solution.

// Problem statement mentions integers, but the samples use floating point
// numbers. So I ended up using long doubles. I'm not entirely sure if
// precision is good enough here, I can rewrite using fractions if necessary.
using ld = long double;
constexpr ld EPS = 1e-9;

// Logging utils.
namespace {
	constexpr bool LOG = false;
	void Log() {
		if(LOG) cerr << '\n';
	}
	template <class T, class... S>
	void Log(T t, S... s) {
		if(LOG) cerr << t << ' ', Log(s...);
	}
}

// Geometry utils.
namespace {
	struct P {
		ld x, y;
		P(ld _x = 0, ld _y = 0) : x(_x), y(_y) {}
		P operator+(const P &p) const { return P{x+p.x, y+p.y}; }
		P operator-(const P &p) const { return P{x-p.x, y-p.y}; }
		ld operator*(const P &p) const { return x*p.x + y*p.y; }
		P operator*(ld c) const { return P{x*c, y*c}; }
		P operator/(ld c) const { return P{x/c, y/c}; }
		ld lsq() { return x*x + y*y; }
	};
	bool contains(P c, ld r, P p) {
		return r >= -EPS && P(c.x - p.x, c.y - p.y).lsq() <= r * r + EPS; 
	}
	bool overlap(P p1, ld r1, P p2, ld r2) {
		return contains(p1, r1 + r2, p2);
	}
	ld sq(ld x) { return x*x; }
	// Intersects two circles. Assumes they intersect in the first place! (i.e.
	// use `overlap` first.) Returns false for one intersection, true for two.
	bool intersect(P p1, ld r1, P p2, ld r2, P &o1, P &o2) {
		// Precompute some terms.
		P d = p1 - p2;
		ld dsq = d.lsq();
		ld det = -(dsq - sq(r1+r2)) * (dsq - sq(r1-r2));
		P m = d * (sq(r2) - sq(r1) + dsq);

		// Single intersection point.
		if (det < EPS) {
			o1 = p2 + m / (2.0 * dsq);
			return false;
		}

		// Two intersection points.
		ld sdet = sqrt(det);
		o1.x = p2.x + (m.x - d.y * sdet) / (2.0 * dsq);
		o1.y = p2.y + (m.y + d.x * sdet) / (2.0 * dsq);
		o2.x = p2.x + (m.x + d.y * sdet) / (2.0 * dsq);
		o2.y = p2.y + (m.y - d.x * sdet) / (2.0 * dsq);
		return true;
	}
	// Project a point onto a line.
	P project(P l1, P l2, P p) {
		P u = l2 - l1;
		return l1 + (u * ((u * (p - l1)) / u.lsq()));
	}
}

// A macro to 'do' f for all triples. Because lazy.
#define DO(f) { int i, j, k;	\
	i = 0; j = 1; k = 2; f;		\
	i = 1; j = 2; k = 0; f;		\
	i = 2; j = 0; k = 1; f;		\
}

// Check a partition for validity.
bool check(P *p, ld *r) {
	// For all triples:
	DO({
		// First check if all pairs of circles actually intersect.
		if (!overlap(p[i], r[i], p[j], r[j])) {
			Log("Pair", i, j, "don't intersect.");
			return false;
		}

		// Then check if one of the circles completely contains the other. If
		// so, we can remove it and solve only for the other two.
		if (contains(p[i], r[i] - r[j], p[j])
		 || contains(p[i], r[i] - r[k], p[k])) {
			Log(i, "contains another circle, check if", j, "&", k, "overlap.");
			return overlap(p[j], r[j], p[k], r[k]);
		}
	});
	
	P p1, p2, pr; // declare outside because the macro doesn't like it.
	DO({
		// Next, consider a pair of intersecting circles. If they intersect in
		// a single point we check if this point is in the third circle. If
		// they intersect in two points, we take the line segment through those
		// points and check if it intersects the third circle.
		// Claim: This is true for at least one pair of circles iff the three
		// circles have a common intersection.
		if (!intersect(p[i], r[i], p[j], r[j], p1, p2)) {
			if (contains(p[k], r[k], p1)) {
				Log("Single point in", p1.x, p1.y);
				return true;
			}
		} else {
			// We want to know if the segment between (x1, y1) and (x2, y2)
			// intersects circle k. First, check if one of the points is
			// inside:
			if (contains(p[k], r[k], p1)
			 || contains(p[k], r[k], p2)) {
				Log("Double point container in", p1.x, p1.y, "or",
					p2.x, p2.y);
				return true;
			}
			
			// This didn't work, so project x_k onto the segment, and check if
			// that point is inside the circles.
			pr = project(p1, p2, p[k]);
			bool con = true;
			for (int l = 0; l < 3; ++l)
				con = con && contains(p[l], r[l], pr);
			if (con) {
				Log("Double point projector in", pr.x, pr.y);
				return true;
			}
		}
	});

	// Nothing found.
	return false;
}

int main() {
	P p[3];
	for (int i = 0; i < 3; ++i)
		cin >> p[i].x >> p[i].y;
	
	int N;
	cin >> N;
	vector<ld> s(N, 0.0);
	for (ld &t : s) cin >> t, t *= 2;

	sort(s.begin(), s.end());
	ld l = s.back() / 2;
	Log("Central radius is", l);
	s.pop_back();

	// Enumerate and check all 3-partitions.
	bool possible = false;
	ld r[3];
	for (int m = 0; m < (1 << (N-1)) && !possible; ++m) {
		int m2 = m;
		do {
			r[0] = r[1] = r[2] = l;
			for (int i = 0; i < N - 1; ++i) {
				if (!((m>>i)&1))
					r[0] += s[i];
				else if (!((m2>>i)&1))
					r[1] += s[i];
				else
					r[2] += s[i];
			}
			possible = possible || check(p, r);
			if (possible) {
				Log("Found:", m, m2);
				Log("Radii:", r[0], r[1], r[2]);
			}
			m2 = (m2 - 1) & m;
		} while (m != m2 && !possible);
	}
	
	// Special case for N=1.
	if (!possible) {
		r[0] = r[1] = r[2] = l;
		possible |= check(p, r);
	}

	cout << (possible ? "possible" : "impossible") << endl;
	return 0;
}
