#!/usr/bin/env perl

use strict;
use warnings;

srand 0;

use Algorithm::Munkres;
use List::Util qw(shuffle);

my $n = int <STDIN>;
die 'Board is too large to draw!' if $n > 26;
my $maxcol = chr($n - 1 + ord 'a');

print <<CHESS;
\\documentclass[class=article,border=0pt]{standalone}
\\usepackage[LSBC3,T1]{fontenc}
\\usepackage[svgnames]{xcolor}
\\usepackage{chessboard}
\\usepackage{fontspec}
\\usepackage{color}

\\begin{document}

\\chessboard[
  boardfontencoding=LSBC3,
%  boardfontfamily=skaknew,
  label=false,
  blackfieldmaskcolor=black!30, blackfieldcolor=black!30,
  whitefieldmaskcolor=black!0,     whitefieldcolor=black!0,
  blackpiececolor=black,
  blackonwhitepiecemaskcolor=white,
  blackonblackpiecemaskcolor=white,
  addfontcolors,
  pgf=true, pgfstyle=knightmove, arrow=latex, linewidth=0.15ex, color=cyan!50!black,
  maxfield=${maxcol}{$n},
CHESS

sub max ($$) { $_[$_[0] < $_[1]] }
sub min ($$) { $_[$_[0] > $_[1]] }

sub coord {
  my ($x, $y) = @_ if @_ == 2 or die;
  return chr($x - 1 + ord 'a') . $y;
}

my @left_pieces = ();
my @right_pieces = ();

my @pieces = ();
my @moves = ();

for my $i (1 ... $n) {
  my ($x, $y) = map {int} split /\s+/, <STDIN>;
  push @right_pieces, {x => $x, y => $y, i => $i};

  push @pieces, 'K' . coord($x, $y);
}
@right_pieces = shuffle @right_pieces;

sub add_move {
  my ($ax, $ay, $bx, $by) = @_;
  while (($ax != $bx) || ($ay != $by)) {
    # print "add_move $ax,$ay $bx,$by\n";
    my ($cx, $cy) = ($ax, $ay);
    if ($ay != $by) {
#    if (abs($ax - $bx) <= int rand (abs($ax - $bx) + ($ay - $by))) {
      $cy = $ay + ($by > $ay ? +1 : -1);
    } else {
      $cx = $ax + ($bx > $ax ? +1 : -1);
    }
    push @moves, (coord($ax, $ay) . '-' . coord($cx, $cy));
    ($ax, $ay) = ($cx, $cy);
  }
}

# Ugly O(N^2) solution, good enough for visualisation under 30 elements.
sub solve1 {
  my $x = 1;
  my $answer = 0;
  for my $i (1 ... $n) {
    while ($x <= $i || @left_pieces == 0) {
      @left_pieces = (@left_pieces, grep {min($_->{x}, $_->{y}) == $x} @right_pieces); 
      ++$x;
    }
    @left_pieces = sort {max($a->{x}, $a->{y}) <=> max($b->{x}, $b->{y})} @left_pieces;
    my $item = shift @left_pieces;
    if ($item->{x} != $i || $item->{y} != $i) {
      add_move $item->{x}, $item->{y}, $i, $i;
    }

    $answer += abs($item->{x} - $i) + abs($item->{y} - $i);
  }
 return $answer; 
}

# Hungarian algorithm. Slow but easy to prove that it's correct. Importantly
# the answers calculated are not necessarily the same as the ones generated by
# a greedy algorithm, which helps to avoid accidentally giving away hints as to
# one of the correct strategies.
sub solve2 {
  my @cost = ();
  for my $i (0 ... $n-1) {
    push @cost, [(0) x $n];
    for my $j (1 ... $n) {
      $cost[$i]->[$j-1] = abs($right_pieces[$i]->{x} - $j) + abs($right_pieces[$i]->{y} - $j);
    }
  }

  my @assignment = (0) x $n;
  assign(\@cost, \@assignment);
  
  my $answer = 0;
  for my $i (0 ... $n-1) {
    add_move
        $right_pieces[$i]->{x}, $right_pieces[$i]->{y},
        $assignment[$i]+1, $assignment[$i]+1;
    $answer += $cost[$i]->[$assignment[$i]];
  }
  return $answer;
}

my $answer = solve2;
print '  markmoves={' . (join ',', @moves) . "},\n";
print '  setblack={' . (join ',', @pieces) . "},\n";

print <<CHESS;
  showmover=false
]
\\end{document}
CHESS

print STDERR "answer = $answer\n";
